
# q-expansion of newform 6552.2.a.p, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6552
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [0], [1], [2], [-1], [6], [-8], [4], [6], [4], [-2], [0], [4], [6], [-6], [-10], [10], [-4], [6], [6], [0], [-6], [12], [-2], [14], [8], [0], [-6], [10], [0], [0], [-8], [20], [-12], [-4], [-18], [8], [-2], [22], [-4], [6], [8], [10], [0], [-16], [4], [4], [-18], [-6], [26], [6], [22], [-16], [18], [-4], [30], [0], [-18], [8], [28], [12], [20], [-4], [-10], [-8], [4], [-2], [12], [30], [16], [-10], [-24], [-6], [-32], [-14], [18], [-18], [36], [30], [16], [10], [-6], [14], [-16], [24], [12], [-34], [8], [-8], [-24], [2], [4], [32], [-28], [-36], [24], [-22], [-36], [10], [44], [-36], [-12], [-22], [-12], [18], [-14], [20], [-12], [38], [24], [22], [32], [44], [-12], [-2], [44], [32], [-10], [-36], [-34], [-14], [-22], [-14], [-28], [50], [50], [36], [-48], [34], [4], [38], [-48], [-22], [-48], [-38], [-20], [-28], [18], [-30], [8], [-36], [-48], [-22], [34], [-30], [14], [18], [4], [26], [-34], [-38], [20], [0], [-44], [8], [-40], [16], [-2], [12], [-42], [18], [-4], [36], [52], [-46], [-48], [-26], [-10], [6], [-42], [30], [52], [62], [8], [6], [-52], [-12], [-8], [-62], [-48], [-12], [58], [36], [22], [-6], [-18], [0], [-42], [46], [-6], [-22], [-20], [28], [-28], [-6], [-14], [-46], [-36], [-24], [16], [-16], [2], [26], [-10], [-70], [28], [48], [-20], [-28], [-2], [-30], [16], [-38], [-6], [-62], [-64], [30], [62], [-28], [-10], [16], [-20], [-12], [8], [-10], [-42], [20], [64], [-34], [14], [-44], [32], [6], [36], [-66], [-58], [-60], [-20], [-24], [26], [76], [56], [-42], [44], [44], [32], [-18], [-16], [-20], [-22], [-32], [-70], [70], [-74], [18], [-54], [-32], [-6], [-14], [16], [60], [-14], [-66], [36], [44], [-72], [-48], [-4], [-42], [-62], [60], [14], [48], [30], [-80], [6], [-74], [2], [8], [16], [-8], [72], [-10], [68], [-24], [-2], [36], [-44], [50], [-54], [-4], [-12], [-2], [-2], [42], [-64], [-6], [60], [-60], [26], [8], [32], [-44], [68], [62], [36], [-34], [-54], [82], [-68], [-16], [-62], [20], [-50], [38], [54], [-30], [70], [38], [-44], [58], [82], [-44], [-4], [-34], [-56], [4], [24], [-26], [-78], [78], [40], [-86], [-80], [-14], [22], [72], [14], [-48], [-50], [50], [44], [32], [-72], [-84], [90], [64], [70], [-22], [-68], [-42], [-4], [16], [-6], [-14], [-34], [-34], [34], [-30], [6], [-26], [20], [-26], [60], [26], [-16], [-36], [-6], [88], [-44], [6], [-78], [-8], [46], [98], [-32], [-46], [-34], [10], [20], [48], [-8], [-68], [8], [-70], [40], [42], [-24], [76], [-42], [-36], [42], [-4], [-50], [-26], [-40], [86], [52], [88], [-10], [-90], [56], [12], [80], [32], [-46], [-60], [92], [-30], [50], [18], [44], [92], [98], [-30], [62], [96], [68], [36], [-26], [70], [-34], [32], [-14], [100], [-8], [36], [-64], [0]]
